import yaml
import pandas as pd
import pyarrow as pa
from typing import Any
from pathlib import Path
import pyarrow.parquet as pq
from .types_check import _path


def get_dirs(path: str | Path) -> list[Path]:
    if not _path(path):
        return []
    return [x for x in Path(path).glob("*") if x.is_dir()]


def get_files(path: str | Path, extension: str | None = None) -> list[Path]:
    if not _path(path):
        return []
    if not extension:
        return [x for x in Path(path).glob("*") if x.is_file()]
    return [x for x in Path(path).glob(f"*.{extension}") if x.is_file()]


def _create(path: str | Path):
    Path(path).parent.mkdir(parents=True, exist_ok=True)


def _load_parquet(content: Any):
    if not isinstance(content, dict):
        return content
    return {key: list(value) for key, value in content.items()}


def load_parquet(path: str):
    """Load a Parquet file."""
    table = pq.read_table(path).to_pandas().to_dict(orient="records")
    return [_load_parquet(x) for x in table]


def save_parquet(path: str, data: list | dict) -> None:
    """Save a list to a Parquet file."""
    _create(path)
    table = pa.Table.from_pandas(pd.DataFrame(data))
    pq.write_table(table, path)


def load_text(
    path: Path | str,
    encoding: str = "utf-8",
) -> str:
    """Load text content from a file. If not exists it returns a empty string."""
    if not _path(path):
        return ""
    with open(path, "r", encoding=encoding) as file:
        return file.read()


def save_text(path: Path | str, content: str, encoding: str = "utf-8") -> None:
    """Save a text file to the provided path."""
    _create(path)
    with open(path, "w", encoding=encoding) as file:
        file.write(content)


def load_yaml(
    path: Path | str,
    encoding: str = "utf-8",
    unsafe_loader=False,
) -> None | list[Any] | dict[Any, Any]:
    """Load YAML content from a file."""
    if not _path(path):
        return None
    with open(path, "r", encoding=encoding) as file:
        if not unsafe_loader:
            return yaml.safe_load(file, Loader=yaml.FullLoader)
        return yaml.unsafe_load(file, Loader=yaml.FullLoader)


def save_yaml(
    path: Path | str,
    content: list | tuple | dict,
    encoding: str = "utf-8",
    safe_dump: bool = False,
) -> None:
    """Save a YAML file to the provided path."""
    _create(path)
    with open(path, "w", encoding=encoding) as file:
        if safe_dump:
            yaml.safe_dump(content, file, encoding=encoding)
        else:
            yaml.dump(content, file, encoding=encoding)



import re
import pyperclip
from datetime import datetime
from .types_check import _array, _dict, _numpy
from typing import Any, TypeAlias, Literal, Callable




def np_list(content):
    if _numpy(content):
        return content.flatten().tolist()


def current_time():
    return f"{datetime.now().strftime('%Y-%m-%d-%H%M%S')}"


def clipboard(text: str):
    """
    Set the clipboard to the given text.
    """
    pyperclip.copy(str(text))


def flatten_list(entry):
    """
    Example:
    ```py
    from grtoolbox.type_tools import flatten_list

    sample = ["test", [[[1]], [2]], 3, [{"last":4}]]
    results = flatten_list(sample)
    # results = ["test", 1, 2, 3, {"last": 4}]
    ```"""
    if _array(entry):
        return [item for sublist in entry for item in flatten_list(sublist)]
    return [entry] if entry is not None else []


def filter_list(entry: list | tuple, types: TypeAlias) -> list:
    if not _array(entry, allow_empty=False):
        return []
    return [x for x in entry if isinstance(x, types)]


def dict_to_list(
    entry: dict[str, Any],
    return_item: Literal["key", "content"] = "content",
) -> list:
    res = []
    assert _dict(entry), "the entry provided is not a valid dictionary. Received: {}".format(entry)
    if return_item == "content":
        return list(entry.values())
    return list(entry.keys())


def try_call(comp: Callable, verbose_exception: bool = False, **kwargs):
    try: return comp(**kwargs)
    except Exception as e:
        if verbose_exception: 
            print(e)


def recursive_replacer(text: str, dic: dict) -> str:
    for i, j in dic.items():
        text = text.replace(i, j)
    return text


def atoi(text: str):
    """Credits"""
    text = text.strip()
    return int(text) if text.isdigit() else text.lower()


def natural_keys(text):
    # From oobabooga
    return [atoi(c) for c in re.split(r"(\d+)", text)]



from pathlib import Path
from numpy import ndarray
from typing import Any, Callable



def _int(entry: Any, check_string: bool = False):
    if check_string and _str(entry):
        try:
            int(entry)
            return True
        except:
            pass
    return isinstance(entry, int)


def _float(entry: Any, check_string: bool = False):
    if check_string and _str(entry):
        try:
            float(entry)
            return True
        except:
            pass
    return isinstance(entry, float)


def _number(entry: Any, check_string: bool = False):
    return any((_int(entry, check_string), _float(entry, check_string)))


def _numpy(entry: Any, allow_empty:bool=True) -> bool:
    return isinstance(entry, ndarray) and (allow_empty or (len(entry.flatten().tolist())))


def _str(
    entry: Any,
    allow_empty: bool = False,
) -> bool:
    if isinstance(entry, (str, Path)):
        entry = str(entry)
        if allow_empty:
            return bool(entry)
        return bool(entry.strip())
    return False


def _array(entry: Any, allow_empty: bool = False):
    if allow_empty or not isinstance(entry, (list, tuple)):
        return isinstance(entry, (list, tuple))
    return bool(entry)


def _dict(entry: Any, allow_empty: bool = False):
    if allow_empty or not isinstance(entry, dict):
        return isinstance(entry, dict)
    return bool(entry)


def _compare(arg1: Any | None, arg2: Any) -> Any:
    """
    arg1 if its not None or arg2.
    It considers falsely values from the first arg.
    Falsely are usualy: (0, "", {}, False, [])
    """
    return arg1 if arg1 is not None else arg2


def _path(entry) -> bool:
    if _str(entry):
        return Path(entry).exists()
    return False



import sys
from time import sleep

from textblob import TextBlob


def split_quotations(text: str):
    results = []
    current = ""
    quotations = text.count('"')
    if quotations > 0:
        for char in text:
            if char == "\n":
                results.append("\n")
            elif char == '"':
                if not current:
                    current = char
                elif current.count('"') % 2 == 0:
                    results.append(current)
                    current = char
                else:
                    current += char
                    results.append(current)
                    current = ""
            else:
                current += char
    else:
        return [text]
    if current and current.count('"') == 0:
        results.append(current)
    return results


def text_spliter(text: str):
    text: list[str] = [x for x in text.replace("\n\n\n", "\n\n").splitlines()]
    preprocess = split_quotations("\n".join(text))
    response = []
    for entry in preprocess:
        if entry.find('"') == -1:
            if entry == "\n":
                response.append(entry)
            else:
                response.extend([x for x in TextBlob(entry).raw_sentences])
        else:
            response.append(entry)

    if len(response) > 1:
        last_text: str = response[-1]
        if last_text.find('"') != -1:
            pass
        elif not last_text.endswith((".", "!", "?")):
            response.pop()
    return response


def split_sentences(text):
    blob = TextBlob(text).raw_sentences
    joined_sentences = []

    # Check if a quotation is not completed and join sentences accordingly
    inquote = False
    for i, sentence in enumerate(blob):
        num_quotes = sentence.count('"')
        if num_quotes % 2 != 0:
            inquote = True
        if inquote:
            if i + 1 < len(blob):
                next_sentence = blob[i + 1]
                sentence += " " + next_sentence
                inquote = False
        joined_sentences.append(sentence)
    if len(joined_sentences) > 1:
        if not joined_sentences[-1].endswith((".", "?", "!", '"')):
            joined_sentences.pop()
    return joined_sentences


[
    "Alexander looked at the knights, his eyes filled with determination.",
    '"I\'m not a traitor," he said, his voice shaking.',
    "Killbride's eyes twinkled with amusement and 3.33 he cocked his head to one side.",
    '"No? Then what were you doing last night, my lord?',
    "Then what were you doing last night, my lord?",
    'Or should I say, what you failed to do." Alexander frowned, knowing he had indeed done nothing.',
    "Alexander frowned, knowing he had indeed done nothing.",
    '"Keep silent!"',
    "Snivel cried, stamping his foot on the ground.",
    "\"You'll get us all killed. I think Lord Kilbride's taking a rather... unconventional approach.\"",
    "I think Lord Kilbride's taking a rather... unconventional approach.\" Lady Yennen gasped in surprise and frowned at her cousin.",
    "Lady Yennen gasped in surprise and frowned at her cousin.",
    '"Snivel, that\'s not like you."',
]

text__ = """Alexander looked at the knights, his eyes filled with determination. "I'm not a traitor," he said, his voice shaking. 

Killbride's eyes twinkled with amusement and 3.33 he cocked his head to one side. "No? Then what were you doing last night, my lord? Or should I say, what you failed to do."

Alexander frowned, knowing he had indeed done nothing. "Keep silent!" Snivel cried, stamping his foot on the ground. "You'll get us all killed. I think Lord Kilbride's taking a rather... unconventional approach."

Lady Yennen gasped in surprise and frowned at her cousin."Snivel, that's not like you."

Snivel pointed at Alexander"""

# tx = text_spliter(text__)
# tx = [x for x in TextBlob(text__).raw_sentences]
tx = split_sentences(text__)
print(tx)
f_text = ""
# for _res in tx:
#     if len(f_text) > 1 and f_text[-2:] != "\n":
#         f_text += " "  + _res
#     else:
#         f_text += _res
print()
print()
print(" ".join(tx))


____ = [
    "Alexander looked at the knights, his eyes filled with determination.",
    '"I\'m not a traitor,"',
    "\n",
    "\n",
    "he said, his voice shaking.Killbride's eyes twinkled with amusement and he cocked his head to one side.",
    '"No? Then what were you doing last night, my lord? Or should I say, what you failed to do."',
    "\n",
    "\n",
    "Alexander frowned, knowing he had indeed done nothing.",
    '"Keep silent!"',
    "Snivel cried, stamping his foot on the ground.",
    "\"You'll get us all killed. I think Lord Kilbride's taking a rather... unconventional approach.\"",
    "\n",
    "\n",
    "Lady Yennen gasped in surprise and frowned at her cousin.",
    '"Snivel, that\'s not like you."',
    "\n",
    "\n",
]

# print(list(text__))



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    version="0.0.3",
    name="gr1336_toolbox",
    description="Personal collection of reusable tools in python.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gr1336/gr1336_toolbox/",
    install_requires=["numpy", "pyperclip", "textblob", "pyyaml", "pyarrow"],
    author="gr1336",
    license=" Apache Software License",
    classifiers=[
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Pre-processors",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
    ],
)